package cd.casic.ci.process.engine.worker;


import cd.casic.ci.process.common.WorkAtom;
import cd.casic.ci.process.dto.req.sast.SastApplicationCreateReq;
import cd.casic.ci.process.dto.req.sast.SastReportCreateReq;
import cd.casic.ci.process.dto.resp.sast.SastApplicationCreateResp;
import cd.casic.ci.process.dto.resp.sast.SastApplicationEchoResp;
import cd.casic.ci.process.dto.resp.sast.SastApplicationStatusResp;
import cd.casic.ci.process.engine.constant.SastConstant;
import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.engine.worker.base.BaseWorker;
import cd.casic.ci.process.process.converter.SastConverter;
import cd.casic.ci.process.process.dataObject.base.PipBaseElement;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.sast.SastService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import com.alibaba.excel.util.StringUtils;
import com.alibaba.fastjson.JSON;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * sast
 * */
@Slf4j
@WorkAtom(taskType = "SAST")
public class SastWorker extends BaseWorker {
    @Resource
    private SastService sastService;
    @Resource
    private SastConverter converter;
    @Override
    public void execute(TaskRunContext context) {
        PipBaseElement contextDef = context.getContextDef();
        String id = contextDef.getId();
        log.info("==============触发worker执行========");
        log.info("==========运行context：{}===========", JSON.toJSONString(context));
        if (contextDef instanceof PipTask task) {
            String applicationId = (String)task.getTaskProperties().get("applicationId");
            if (StringUtils.isEmpty(applicationId)) {
                append(context,"未获取到sast信息");
                toBadEnding();
            }
            SastApplicationEchoResp applicationEcho = sastService.getApplicationEcho(applicationId);
            append(context,"获取到sast任务配置"+JSON.toJSONString(applicationEcho));
            SastApplicationCreateReq sastApplicationCreateReq = converter.converter(applicationEcho);
            List<String> list = applicationEcho.getLodeFiles().stream().map(SastApplicationEchoResp.LodeFile::getFileId).toList();
            sastApplicationCreateReq.setFileId(list);
            append(context,"开始启动SAST任务,检测文件信息：{}"+JSON.toJSONString(applicationEcho.getLodeFiles()));
            log.info("SAST启动任务入参:{}",JSON.toJSONString(sastApplicationCreateReq));
            SastApplicationCreateResp sastApplicationCreateResp = sastService.applicationCreate(sastApplicationCreateReq);
            log.info("SAST启动任务返回值:{}",JSON.toJSONString(sastApplicationCreateResp));
            append(context,"启动任务完毕");
            // 检测状态是否完毕
            SastApplicationStatusResp applicationStatus = sastService.getApplicationStatus(applicationId);
            while (!SastConstant.JOB_DONE.equals(applicationStatus.getJobStatus())) {
                try {
                    Thread.sleep(1000L);
                } catch (InterruptedException e) {
                    log.error("SAST WORKER线程中断");
                }
                applicationStatus=sastService.getApplicationStatus(applicationId);
                append(context,"当前SAST运行中，运行状态："+applicationStatus.getJobStatus());
            }
            String reportJSON = getReportJSON(applicationId, SastConstant.REPORT_MODE_DETAILS);
            String reportUrl = getReportUrl(applicationId, SastConstant.REPORT_MODE_DETAILS);
            task.getTaskProperties().put(SastConstant.REPORT_JSON,reportJSON);
            task.getTaskProperties().put(SastConstant.REPORT_URL,reportUrl);
        }
    }
    private String getReportJSON(String applicationId,String model){
        String reqJSON = "{\"mode\":\"DETAILS\",\"contacts\":\"admin@clouditera.com\",\"reportType\":\"APPLICATION\",\"format\":\"JSON\",\"projectId\":\"8a863857-bab3-40d9-a79a-2cb6617a9dd3\",\"applicationIds\":[\"9e3e55d6-6d28-4fd1-adc2-15b6cf0702db\"],\"bugLevels\":[\"DEADLY\",\"SEVERITY\",\"POOR_RISK\",\"LOW_RISK\"],\"audRiskLevels\":[\"WAITING\",\"IGNORE\",\"HIGH\",\"MEDIUM\",\"LOW\",\"NONE\"],\"applicationScope\":\"ALL\"}";
        SastReportCreateReq sastReportCreateReq = JSON.parseObject(reqJSON, SastReportCreateReq.class);
        sastReportCreateReq.setMode(model);
        sastReportCreateReq.setFormat(SastConstant.REPORT_FORMAT_JSON);
        sastReportCreateReq.setApplicationIds(Collections.singletonList(applicationId));
        String jsonReportUrl = sastService.reportIndex(sastReportCreateReq);
        return getJSONString(jsonReportUrl);
    }
    private String getReportUrl(String applicationId,String model){
        String reqJSON = "{\"mode\":\"DETAILS\",\"contacts\":\"admin@clouditera.com\",\"reportType\":\"APPLICATION\",\"format\":\"JSON\",\"projectId\":\"8a863857-bab3-40d9-a79a-2cb6617a9dd3\",\"applicationIds\":[\"9e3e55d6-6d28-4fd1-adc2-15b6cf0702db\"],\"bugLevels\":[\"DEADLY\",\"SEVERITY\",\"POOR_RISK\",\"LOW_RISK\"],\"audRiskLevels\":[\"WAITING\",\"IGNORE\",\"HIGH\",\"MEDIUM\",\"LOW\",\"NONE\"],\"applicationScope\":\"ALL\"}";
        SastReportCreateReq sastReportCreateReq = JSON.parseObject(reqJSON, SastReportCreateReq.class);
        sastReportCreateReq.setMode(model);
        sastReportCreateReq.setFormat(SastConstant.REPORT_FORMAT_PDF);
        sastReportCreateReq.setApplicationIds(Collections.singletonList(applicationId));
        return sastService.reportIndex(sastReportCreateReq);
    }
    private String getJSONString(String reportId){
        String s = sastService.reportDownload(reportId);
        URI uri = URI.create(s);
        URL url = null;
        try {
            url = uri.toURL();
        } catch (MalformedURLException e) {
            log.error("获取json格式报告失败");
        }
        try (InputStream in = url.openStream()) {
            byte[] bytes = in.readAllBytes();
            return new String(bytes, StandardCharsets.UTF_8);
        } catch (IOException e) {
            log.error("获取json格式报告失败");
            throw new ServiceException(GlobalErrorCodeConstants.PIPELINE_ERROR.getCode(),"获取JSON格式报告失败");
        }
    }
}
